<!DOCTYPE html>
<html lang="{% if lang %}{{ lang }}{% else %}en{% endif %}">

<head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{% if title %}{{ title }}{% endif %}</title>
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/water.css">
</head>

<body>
    <main>{% block content %}{% endblock content %}</main>
</body>

</html>