<div style="border: 1px solid blue;">
  <p>
    This is another partial: {{ now|date:"c" }}
  </p>

  <p>
    Under the hood, it gets pulled in with
    <dj-verbatim>{% include %}</dj-verbatim>
  </p>
</div>